from .test_module_bucket_aws import *


class TestBucket_DigitalOcean(Bucket_AWS_Base):
    provider = "digitalocean"
    random_bucket_1 = f"{random_bucket_name_1}.fra1.digitaloceanspaces.com"
    random_bucket_2 = f"{random_bucket_name_2}.fra1.digitaloceanspaces.com"
    random_bucket_3 = f"{random_bucket_name_3}.fra1.digitaloceanspaces.com"

    open_bucket_body = """<?xml version="1.0" encoding="UTF-8"?><ListBucketResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/"><Name>cloud01</Name><Prefix></Prefix><MaxKeys>1000</MaxKeys><IsTruncated>false</IsTruncated><Contents><Key>test.doc</Key><LastModified>2020-10-14T15:23:37.545Z</LastModified><ETag>&quot;4d25c8699f7347acc9f41e57148c62c0&quot;</ETag><Size>13362425</Size><StorageClass>STANDARD</StorageClass><Owner><ID>1957883</ID><DisplayName>1957883</DisplayName></Owner><Type>Normal</Type></Contents><Marker></Marker></ListBucketResult>"""



from .crobat import crobat


class urlscan(crobat):
    flags = ["subdomain-enum", "passive", "safe"]
    watched_events = ["DNS_NAME"]
    produced_events = ["DNS_NAME", "URL_UNVERIFIED"]
    meta = {
        "description": "Query urlscan.io for subdomains",
    }
    options = {"urls": False}
    options_desc = {"urls": "Emit URLs in addition to DNS_NAMEs"}

    base_url = "https://urlscan.io/api/v1"

    async def setup(self):
        self.urls = self.config.get("urls", False)
        return await super().setup()

    async def handle_event(self, event):
        query = self.make_query(event)
        for domain, url in await self.query(query):
            source_event = event
            if domain and domain != query:
                domain_event = self.make_event(domain, "DNS_NAME", source=event)
                if domain_event:
                    if str(domain_event.host).endswith(query) and not str(domain_event.host) == str(event.host):
                        self.emit_event(domain_event, abort_if=self.abort_if)
                        source_event = domain_event
            if url:
                url_event = self.make_event(url, "URL_UNVERIFIED", source=source_event)
                if url_event:
                    if str(url_event.host).endswith(query):
                        if self.urls:
                            self.emit_event(url_event, abort_if=self.abort_if)
                        else:
                            self.emit_event(str(url_event.host), "DNS_NAME", source=event, abort_if=self.abort_if)
                    else:
                        self.debug(f"{url_event.host} does not match {query}")

    async def query(self, query):
        results = set()
        url = f"{self.base_url}/search/?q={self.helpers.quote(query)}"
        r = await self.helpers.request(url)
        try:
            json = r.json()
            if json and type(json) == dict:
                for result in json.get("results", []):
                    if result and type(result) == dict:
                        task = result.get("task", {})
                        if task and type(task) == dict:
                            domain = task.get("domain", "")
                            url = task.get("url", "")
                            if domain or url:
                                results.add((domain, url))
                        page = result.get("page", {})
                        if page and type(page) == dict:
                            domain = page.get("domain", "")
                            url = page.get("url", "")
                            if domain or url:
                                results.add((domain, url))
            else:
                self.debug(f'No results for "{query}"')
        except Exception:
            self.verbose(f"Error retrieving urlscan results")
        return results



from .test_module_paramminer_headers import *


class TestParamminer_Getparams(TestParamminer_Headers):
    modules_overrides = ["httpx", "paramminer_getparams"]
    config_overrides = {"modules": {"paramminer_getparams": {"wordlist": tempwordlist(["canary", "id"])}}}

    getparam_body = """
    <html>
    <title>the title</title>
    <body>
    <p>Hello null!</p>';
    </body>
    </html>
    """

    getparam_body_match = """
    <html>
    <title>the title</title>
    <body>
    <p>Hello AAAAAAAAAAAAAA!</p>';
    </body>
    </html>
    """

    async def setup_after_prep(self, module_test):
        module_test.scan.modules["paramminer_getparams"].rand_string = lambda *args, **kwargs: "AAAAAAAAAAAAAA"
        module_test.monkeypatch.setattr(
            helper.HttpCompare, "gen_cache_buster", lambda *args, **kwargs: {"AAAAAA": "1"}
        )
        expect_args = {"query_string": b"id=AAAAAAAAAAAAAA&AAAAAA=1"}
        respond_args = {"response_data": self.getparam_body_match}
        module_test.set_expect_requests(expect_args=expect_args, respond_args=respond_args)

        respond_args = {"response_data": self.getparam_body}
        module_test.set_expect_requests(respond_args=respond_args)

    def check(self, module_test, events):
        assert any(
            e.type == "FINDING"
            and "[Paramminer] Getparam: [id] Reasons: [body] Reflection: [True]" in e.data["description"]
            for e in events
        )
        assert not any(
            e.type == "FINDING" and "[Paramminer] Getparam: [canary] Reasons: [body]" in e.data["description"]
            for e in events
        )


class TestParamminer_Getparams_noreflection(TestParamminer_Getparams):
    getparam_body_match = """
    <html>
    <title>the title</title>
    <body>
    <p>Hello ADMINISTRATOR!</p>';
    </body>
    </html>
    """

    def check(self, module_test, events):
        assert any(
            e.type == "FINDING"
            and "[Paramminer] Getparam: [id] Reasons: [body] Reflection: [False]" in e.data["description"]
            for e in events
        )


class TestParamminer_Getparams_boring_off(TestParamminer_Getparams):
    config_overrides = {
        "modules": {
            "paramminer_getparams": {"skip_boring_words": False, "wordlist": tempwordlist(["canary", "boring"])}
        }
    }

    async def setup_after_prep(self, module_test):
        module_test.scan.modules["paramminer_getparams"].rand_string = lambda *args, **kwargs: "AAAAAAAAAAAAAA"
        module_test.scan.modules["paramminer_getparams"].boringlist = ["boring"]
        module_test.monkeypatch.setattr(
            helper.HttpCompare, "gen_cache_buster", lambda *args, **kwargs: {"AAAAAA": "1"}
        )
        expect_args = {"query_string": b"boring=AAAAAAAAAAAAAA&AAAAAA=1"}
        respond_args = {"response_data": self.getparam_body_match}
        module_test.set_expect_requests(expect_args=expect_args, respond_args=respond_args)

        respond_args = {"response_data": self.getparam_body}
        module_test.set_expect_requests(respond_args=respond_args)

    def check(self, module_test, events):
        assert any(
            e.type == "FINDING" and "[Paramminer] Getparam: [boring] Reasons: [body]" in e.data["description"]
            for e in events
        )


class TestParamminer_Getparams_boring_on(TestParamminer_Getparams_boring_off):
    config_overrides = {
        "modules": {
            "paramminer_getparams": {"skip_boring_words": True, "wordlist": tempwordlist(["canary", "boring"])}
        }
    }

    def check(self, module_test, events):
        assert not any(
            e.type == "FINDING" and "[Paramminer] Getparam: [boring] Reasons: [body]" in e.data["description"]
            for e in events
        )


class TestParamminer_Getparams_Extract_Json(TestParamminer_Headers):
    modules_overrides = ["httpx", "paramminer_getparams"]
    config_overrides = {
        "modules": {"paramminer_getparams": {"wordlist": tempwordlist(["canary", "id"]), "http_extract": True}}
    }

    getparam_extract_json = """
    {
  "obscureParameter": 1,
  "common": 1
}
    """

    getparam_extract_json_match = """
    {
  "obscureParameter": "AAAAAAAAAAAAAA",
  "common": 1
}
    """

    async def setup_after_prep(self, module_test):
        module_test.scan.modules["paramminer_getparams"].rand_string = lambda *args, **kwargs: "AAAAAAAAAAAAAA"
        module_test.monkeypatch.setattr(
            helper.HttpCompare, "gen_cache_buster", lambda *args, **kwargs: {"AAAAAA": "1"}
        )

        expect_args = {"query_string": b"obscureParameter=AAAAAAAAAAAAAA&common=AAAAAAAAAAAAAA&AAAAAA=1"}
        respond_args = {
            "response_data": self.getparam_extract_json_match,
            "headers": {"Content-Type": "application/json"},
        }
        module_test.set_expect_requests(expect_args=expect_args, respond_args=respond_args)

        expect_args = {"query_string": b"obscureParameter=AAAAAAAAAAAAAA&AAAAAA=1"}
        respond_args = {
            "response_data": self.getparam_extract_json_match,
            "headers": {"Content-Type": "application/json"},
        }
        module_test.set_expect_requests(expect_args=expect_args, respond_args=respond_args)

        respond_args = {"response_data": self.getparam_extract_json, "headers": {"Content-Type": "application/json"}}
        module_test.set_expect_requests(respond_args=respond_args)

    def check(self, module_test, events):
        assert any(
            e.type == "FINDING"
            and "[Paramminer] Getparam: [obscureParameter] Reasons: [body]" in e.data["description"]
            for e in events
        )


class TestParamminer_Getparams_Extract_Xml(TestParamminer_Headers):
    modules_overrides = ["httpx", "paramminer_getparams"]
    config_overrides = {"modules": {"paramminer_getparams": {"wordlist": tempwordlist([]), "http_extract": True}}}

    getparam_extract_xml = """
<data>
    <obscureParameter>1</obscureParameter>
    <common>1</common>
</data>
    """

    getparam_extract_xml_match = """
<data>
    <obscureParameter>AAAAAAAAAAAAAA</obscureParameter>
    <common>1</common>
</data>
    """

    async def setup_after_prep(self, module_test):
        module_test.scan.modules["paramminer_getparams"].rand_string = lambda *args, **kwargs: "AAAAAAAAAAAAAA"
        module_test.monkeypatch.setattr(
            helper.HttpCompare, "gen_cache_buster", lambda *args, **kwargs: {"AAAAAA": "1"}
        )

        expect_args = {
            "query_string": b"data=AAAAAAAAAAAAAA&obscureParameter=AAAAAAAAAAAAAA&common=AAAAAAAAAAAAAA&AAAAAA=1"
        }
        respond_args = {
            "response_data": self.getparam_extract_xml_match,
            "headers": {"Content-Type": "application/xml"},
        }
        module_test.set_expect_requests(expect_args=expect_args, respond_args=respond_args)

        expect_args = {"query_string": b"common=AAAAAAAAAAAAAA&obscureParameter=AAAAAAAAAAAAAA&AAAAAA=1"}
        respond_args = {
            "response_data": self.getparam_extract_xml_match,
            "headers": {"Content-Type": "application/xml"},
        }
        module_test.set_expect_requests(expect_args=expect_args, respond_args=respond_args)

        expect_args = {"query_string": b"obscureParameter=AAAAAAAAAAAAAA&AAAAAA=1"}
        respond_args = {
            "response_data": self.getparam_extract_xml_match,
            "headers": {"Content-Type": "application/xml"},
        }
        module_test.set_expect_requests(expect_args=expect_args, respond_args=respond_args)

        respond_args = {"response_data": self.getparam_extract_xml, "headers": {"Content-Type": "application/xml"}}
        module_test.set_expect_requests(respond_args=respond_args)

    def check(self, module_test, events):
        assert any(
            e.type == "FINDING"
            and "[Paramminer] Getparam: [obscureParameter] Reasons: [body]" in e.data["description"]
            for e in events
        )


class TestParamminer_Getparams_Extract_Html(TestParamminer_Headers):
    modules_overrides = ["httpx", "paramminer_getparams"]
    config_overrides = {
        "modules": {"paramminer_getparams": {"wordlist": tempwordlist(["canary"]), "http_extract": True}}
    }

    getparam_extract_html = """
<html><a href="/?hack=1">ping</a></html>
    """

    getparam_extract_html_match = """
<html><a href="/?hack=1">ping</a><p>HackThePlanet</p></html>
    """

    async def setup_after_prep(self, module_test):
        module_test.scan.modules["paramminer_getparams"].rand_string = lambda *args, **kwargs: "AAAAAAAAAAAAAA"
        module_test.monkeypatch.setattr(
            helper.HttpCompare, "gen_cache_buster", lambda *args, **kwargs: {"AAAAAA": "1"}
        )

        expect_args = {"query_string": b"id=AAAAAAAAAAAAAA&hack=AAAAAAAAAAAAAA&AAAAAA=1"}
        respond_args = {"response_data": self.getparam_extract_html_match, "headers": {"Content-Type": "text/html"}}
        module_test.set_expect_requests(expect_args=expect_args, respond_args=respond_args)

        expect_args = {"query_string": b"hack=AAAAAAAAAAAAAA&AAAAAA=1"}
        respond_args = {"response_data": self.getparam_extract_html_match, "headers": {"Content-Type": "text/html"}}
        module_test.set_expect_requests(expect_args=expect_args, respond_args=respond_args)

        respond_args = {"response_data": self.getparam_extract_html, "headers": {"Content-Type": "text/html"}}
        module_test.set_expect_requests(respond_args=respond_args)

    def check(self, module_test, events):
        assert any(
            e.type == "FINDING" and "[Paramminer] Getparam: [hack] Reasons: [body]" in e.data["description"]
            for e in events
        )



from .paramminer_headers import paramminer_headers


class paramminer_cookies(paramminer_headers):
    """
    Inspired by https://github.com/PortSwigger/param-miner
    """

    watched_events = ["HTTP_RESPONSE"]
    produced_events = ["FINDING"]
    flags = ["active", "aggressive", "slow", "web-paramminer"]
    meta = {
        "description": "Smart brute-force to check for common HTTP cookie parameters",
    }
    options = {"wordlist": ""}  # default is defined separately
    options_desc = {"wordlist": "Define the wordlist to be used to derive cookies"}
    scanned_hosts = []
    boringlist = []
    max_event_handlers = 12
    in_scope_only = True
    compare_mode = "cookie"
    default_wordlist = "paramminer_parameters.txt"

    async def check_batch(self, compare_helper, url, cookie_list):
        cookies = {p: self.rand_string(14) for p in cookie_list}
        return await compare_helper.compare(url, cookies=cookies, check_reflection=(len(cookie_list) == 1))

    def gen_count_args(self, url):
        cookie_count = 40
        while 1:
            if cookie_count < 0:
                break
            fake_cookies = {self.rand_string(14): self.rand_string(14) for _ in range(0, cookie_count)}
            yield cookie_count, (url,), {"cookies": fake_cookies}
            cookie_count -= 5



from .base import ModuleTestBase


class TestWebReport(ModuleTestBase):
    targets = ["http://127.0.0.1:8888"]
    modules_overrides = ["httpx", "wappalyzer", "badsecrets", "web_report", "secretsdb"]

    async def setup_before_prep(self, module_test):
        # secretsdb --> FINDING
        module_test.httpx_mock.add_response(
            url="https://raw.githubusercontent.com/blacklanternsecurity/secrets-patterns-db/master/db/rules-stable.yml",
            text="""patterns:
- pattern:
    confidence: 99
    name: Asymmetric Private Key
    regex: '-----BEGIN ((EC|PGP|DSA|RSA|OPENSSH) )?PRIVATE KEY( BLOCK)?-----'""",
        )
        # wappalyzer --> TECHNOLOGY
        # badsecrets --> VULNERABILITY
        respond_args = {"response_data": web_body}
        module_test.set_expect_requests(respond_args=respond_args)

    def check(self, module_test, events):
        report_file = module_test.scan.home / "web_report.html"
        with open(report_file) as f:
            report_content = f.read()
        assert "<li>[HIGH] Known Secret Found" in report_content
        assert (
            """<h3>URL</h3>
<ul>
<li><strong>http://127.0.0.1:8888/</strong>"""
            in report_content
        )
        assert (
            """<h3>FINDING</h3>
<ul>
<li>Possible secret (Asymmetric Private Key)"""
            in report_content
        )
        assert "<h3>TECHNOLOGY</h3>" in report_content
        assert "<p>flask</p>" in report_content


web_body = """
<html>
<body>
<link rel="preconnect" href="https://fonts.googleapis.com">
<link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
<link href="https://fonts.googleapis.com/css2?family=Open+Sans+Condensed:wght@700&family=Open+Sans:ital,wght@0,400;0,600;0,700;0,800;1,400&display=swap" rel="stylesheet">
    <form method="post" action="./query.aspx" id="form1">
<div class="aspNetHidden">
<input type="hidden" name="__VIEWSTATE" id="__VIEWSTATE" value="rJdyYspajyiWEjvZ/SMXsU/1Q6Dp1XZ/19fZCABpGqWu+s7F1F/JT1s9mP9ED44fMkninhDc8eIq7IzSllZeJ9JVUME41i8ozheGunVSaESf4nBu" />
</div>

<div class="aspNetHidden">

    <input type="hidden" name="__VIEWSTATEGENERATOR" id="__VIEWSTATEGENERATOR" value="EDD8C9AE" />
    <input type="hidden" name="__VIEWSTATEENCRYPTED" id="__VIEWSTATEENCRYPTED" value="" />
</div>
    </form>
    <p>-----BEGIN PGP PRIVATE KEY BLOCK-----</p>
</body>
</html>
"""



import re

from .viewdns import viewdns


class azure_tenant(viewdns):
    watched_events = ["DNS_NAME"]
    produced_events = ["DNS_NAME"]
    flags = ["affiliates", "subdomain-enum", "passive", "safe"]
    meta = {"description": "Query Azure for tenant sister domains"}

    base_url = "https://autodiscover-s.outlook.com"
    in_scope_only = True

    async def setup(self):
        self.processed = set()
        self.d_xml_regex = re.compile(r"<Domain>([^<>/]*)</Domain>", re.I)
        return True

    async def handle_event(self, event):
        _, query = self.helpers.split_domain(event.data)
        domains, _ = await self.query(query)
        if domains:
            self.success(f'Found {len(domains):,} domains under tenant for "{query}"')
        for domain in domains:
            if domain != query:
                self.emit_event(domain, "DNS_NAME", source=event, tags=["affiliate"])
        # todo: tenants?

    async def query(self, domain):
        url = f"{self.base_url}/autodiscover/autodiscover.svc"
        data = f"""<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:exm="http://schemas.microsoft.com/exchange/services/2006/messages" xmlns:ext="http://schemas.microsoft.com/exchange/services/2006/types" xmlns:a="http://www.w3.org/2005/08/addressing" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
    <soap:Header>
        <a:Action soap:mustUnderstand="1">http://schemas.microsoft.com/exchange/2010/Autodiscover/Autodiscover/GetFederationInformation</a:Action>
        <a:To soap:mustUnderstand="1">https://autodiscover-s.outlook.com/autodiscover/autodiscover.svc</a:To>
        <a:ReplyTo>
            <a:Address>http://www.w3.org/2005/08/addressing/anonymous</a:Address>
        </a:ReplyTo>
    </soap:Header>
    <soap:Body>
        <GetFederationInformationRequestMessage xmlns="http://schemas.microsoft.com/exchange/2010/Autodiscover">
            <Request>
                <Domain>{domain}</Domain>
            </Request>
        </GetFederationInformationRequestMessage>
    </soap:Body>
</soap:Envelope>"""

        headers = {
            "Content-Type": "text/xml; charset=utf-8",
            "SOAPAction": '"http://schemas.microsoft.com/exchange/2010/Autodiscover/Autodiscover/GetFederationInformation"',
            "User-Agent": "AutodiscoverClient",
            "Accept-Encoding": "identity",
        }

        self.debug(f"Retrieving tenant domains at {url}")

        r = await self.request_with_fail_count(url, method="POST", headers=headers, data=data)
        status_code = getattr(r, "status_code", 0)
        if status_code not in (200, 421):
            self.warning(f'Error retrieving azure_tenant domains for "{domain}" (status code: {status_code})')
            return set(), set()
        found_domains = list(set(self.d_xml_regex.findall(r.text)))
        domains = set()
        tenantnames = set()

        for d in found_domains:
            # tenant names
            if d.lower().endswith(".onmicrosoft.com"):
                tenantnames.add(d.split(".")[0].lower())
            # make sure we don't make any unnecessary api calls
            d = str(d).lower()
            _, query = self.helpers.split_domain(d)
            self.processed.add(hash(query))
            domains.add(d)
            # absorb into word cloud
            self.scan.word_cloud.absorb_word(d)

        return domains, tenantnames



import os
import time
import logging
from contextlib import suppress
from collections import OrderedDict

from .misc import sha1

log = logging.getLogger("bbot.core.helpers.cache")


def cache_get(self, key, text=True, cache_hrs=24 * 7):
    """
    Get an item from the cache. Default expiration is 1 week.
    Returns None if item is not in cache
    """
    filename = self.cache_filename(key)
    if filename.is_file():
        valid = self.is_cached(key, cache_hrs)
        if valid:
            open_kwargs = {}
            if text:
                open_kwargs.update({"mode": "r", "encoding": "utf-8", "errors": "ignore"})
            else:
                open_kwargs["mode"] = "rb"
            log.debug(f'Using cached content for "{key}"')
            return open(filename, **open_kwargs).read()
        else:
            log.debug(f'Cached content for "{key}" is older than {cache_hrs:,} hours')


def cache_put(self, key, content):
    """
    Put an item in the cache.
    """
    filename = self.cache_filename(key)
    if type(content) == bytes:
        open_kwargs = {"mode": "wb"}
    else:
        open_kwargs = {"mode": "w", "encoding": "utf-8"}
        content = str(content)
    with open(filename, **open_kwargs) as f:
        f.write(content)


def is_cached(self, key, cache_hrs=24 * 7):
    filename = self.cache_filename(key)
    if filename.is_file():
        (m, i, d, n, u, g, sz, atime, mtime, ctime) = os.stat(filename)
        return mtime > time.time() - cache_hrs * 3600
    return False


def cache_filename(self, key):
    return self.cache_dir / sha1(key).hexdigest()


_sentinel = object()


class CacheDict:
    """
    Dictionary to store cached values, with a maximum size limit
    """

    def __init__(self, max_size=1000):
        self._cache = OrderedDict()
        self._max_size = int(max_size)

    def get(self, name, fallback=_sentinel):
        name_hash = self._hash(name)
        try:
            return self._cache[name_hash]
        except KeyError:
            if fallback is not _sentinel:
                return fallback
            raise
        finally:
            with suppress(KeyError):
                self._cache.move_to_end(name_hash)
            self._truncate()

    def put(self, name, value):
        name_hash = self._hash(name)
        try:
            self._cache[name_hash] = value
        finally:
            with suppress(KeyError):
                self._cache.move_to_end(name_hash)
            self._truncate()

    def _truncate(self):
        if not self or len(self) <= self._max_size:
            return
        for nh in list(self._cache.keys()):
            try:
                del self._cache[nh]
            except KeyError:
                pass
            if not self or len(self) <= self._max_size:
                break

    def keys(self):
        return self._cache.keys()

    def values(self):
        return self._cache.values()

    def items(self):
        return self._cache.items()

    def _hash(self, v):
        if type(v) == int:
            return v
        return hash(str(v))

    def __contains__(self, item):
        return self._hash(item) in self._cache

    def __iter__(self):
        return iter(self._cache)

    def __getitem__(self, item):
        return self.get(item)

    def __setitem__(self, item, value):
        self.put(item, value)

    def __bool__(self):
        return bool(self._cache)

    def __len__(self):
        return len(self._cache)



import dns.zone
import dns.query
import dns.message

from .base import ModuleTestBase


class TestDNSZoneTransfer(ModuleTestBase):
    targets = ["blacklanternsecurity.fakedomain"]
    config_overrides = {"dns_resolution": True}

    async def setup_after_prep(self, module_test):
        old_resolve_fn = module_test.scan.helpers.dns._resolve_hostname

        async def _resolve_hostname(query, **kwargs):
            if query == "blacklanternsecurity.fakedomain" and kwargs.get("rdtype", "").upper() == "NS":
                return [module_test.mock_record("ns01.blacklanternsecurity.fakedomain", "NS")], []
            if query == "ns01.blacklanternsecurity.fakedomain" and kwargs.get("rdtype", "").upper() == "A":
                return [module_test.mock_record("127.0.0.1", "A")], []
            return await old_resolve_fn(query, **kwargs)

        def from_xfr(*args, **kwargs):
            zone_text = """
@ 600 IN SOA ns.blacklanternsecurity.fakedomain. admin.blacklanternsecurity.fakedomain. (
    1   ; Serial
    3600   ; Refresh
    900   ; Retry
    604800   ; Expire
    86400 )  ; Minimum TTL
@ 600 IN NS ns.blacklanternsecurity.fakedomain.
@ 600 IN A 127.0.0.1
asdf 600 IN A 127.0.0.1
zzzz 600 IN AAAA dead::beef
"""
            zone = dns.zone.from_text(zone_text, origin="blacklanternsecurity.fakedomain.")
            return zone

        module_test.monkeypatch.setattr("dns.zone.from_xfr", from_xfr)
        module_test.monkeypatch.setattr(module_test.scan.helpers.dns, "_resolve_hostname", _resolve_hostname)

    def check(self, module_test, events):
        assert any(e.data == "asdf.blacklanternsecurity.fakedomain" for e in events), "Zone transfer failed"
        assert any(e.data == "zzzz.blacklanternsecurity.fakedomain" for e in events), "Zone transfer failed"



from bbot.modules.base import BaseModule

common_srvs = [
    # Micro$oft
    "_ldap._tcp.dc._msdcs",
    "_ldap._tcp.gc._msdcs",
    "_ldap._tcp.pdc._msdcs",
    "_ldap._tcp",
    "_ldap._tcp.ForestDNSZones",
    "_gc._msdcs",
    "_kpasswd._tcp",
    "_kpasswd._udp",
    "_kerberos._tcp.dc._msdcs",
    "_kerberos.tcp.dc._msdcs",
    "_kerberos-master._tcp",
    "_kerberos-master._udp",
    "_kerberos._tcp",
    "_kerberos._udp",
    "_autodiscover._tcp",
    # NTP
    "_ntp._udp",
    # mDNS
    "_nntp._tcp",
    # email
    "_imap._tcp",
    "_imap.tcp",
    "_imaps._tcp",
    "_pop3._tcp",
    "_pop3s._tcp",
    "_smtp._tcp",
    # MailEnable
    "_caldav._tcp",
    "_caldavs._tcp",
    "_carddav._tcp",
    "_carddavs._tcp",
    # STUN
    "_stun._tcp",
    "_stun._udp",
    "_stuns._tcp",
    "_turn._tcp",
    "_turn._udp",
    "_turns._tcp",
    # SIP
    "_h323be._tcp",
    "_h323be._udp",
    "_h323cs._tcp",
    "_h323cs._udp",
    "_h323ls._tcp",
    "_h323ls._udp",
    "_sip._tcp",
    "_sip._tls",
    "_sip._udp",
    "_sipfederationtls._tcp",
    "_sipinternal._tcp",
    "_sipinternaltls._tcp",
    "_sips._tcp",
    # misc
    "_aix._tcp",
    "_certificates._tcp",
    "_cmp._tcp",
    "_crl._tcp",
    "_crls._tcp",
    "_finger._tcp",
    "_ftp._tcp",
    "_gc._tcp",
    "_hkp._tcp",
    "_hkps._tcp",
    "_http._tcp",
    "_https._tcp",
    "_jabber-client._tcp",
    "_jabber-client._udp",
    "_jabber._tcp",
    "_jabber._udp",
    "_ocsp._tcp",
    "_pgpkeys._tcp",
    "_pgprevokations._tcp",
    "_PKIXREP._tcp",
    "_submission._tcp",
    "_svcp._tcp",
    "_telnet._tcp",
    "_test._tcp",
    "_whois._tcp",
    "_x-puppet-ca._tcp",
    "_x-puppet._tcp",
    "_xmpp-client._tcp",
    "_xmpp-client._udp",
    "_xmpp-server._tcp",
    "_xmpp-server._udp",
]


class dnscommonsrv(BaseModule):
    watched_events = ["DNS_NAME"]
    produced_events = ["DNS_NAME"]
    flags = ["subdomain-enum", "passive", "safe"]
    meta = {"description": "Check for common SRV records"}
    max_event_handlers = 5

    async def filter_event(self, event):
        # skip SRV wildcards
        if "SRV" in await self.helpers.is_wildcard(event.host):
            return False
        return True

    async def handle_event(self, event):
        queries = [event.data] + [f"{srv}.{event.data}" for srv in common_srvs]
        async for query, results in self.helpers.resolve_batch(queries, type="srv"):
            if results:
                self.emit_event(query, "DNS_NAME", tags=["srv-record"], source=event)



from .base import ModuleTestBase


class TestSecretsDB(ModuleTestBase):
    targets = ["http://127.0.0.1:8888"]
    modules_overrides = ["httpx", "secretsdb"]

    async def setup_before_prep(self, module_test):
        module_test.httpx_mock.add_response(
            url="https://raw.githubusercontent.com/blacklanternsecurity/secrets-patterns-db/master/db/rules-stable.yml",
            text="""patterns:
- pattern:
    confidence: 99
    name: Asymmetric Private Key
    regex: '-----BEGIN ((EC|PGP|DSA|RSA|OPENSSH) )?PRIVATE KEY( BLOCK)?-----'""",
        )
        expect_args = {"method": "GET", "uri": "/"}
        respond_args = {"response_data": "-----BEGIN PGP PRIVATE KEY BLOCK-----"}
        module_test.set_expect_requests(expect_args=expect_args, respond_args=respond_args)

    def check(self, module_test, events):
        assert any(e.type == "FINDING" for e in events)
